OperandStack

Struct OperandStack 

pub struct OperandStack {
    pub values: Vec<Value>,
}
The operand stack

StackAssembly’s evaluation model is based on an implicit stack which stores all operands.

Aside from this, the stack is an important communication channel between script and host. Please refer to Eval’s operand_stack field for more information on that.

Fields

values: Vec<Value>

The values on the stack

Implementations

Source

impl OperandStack

pub fn push(&mut self, value: impl Into<Value>)

Push a value to the top of the stack
pub fn pop(&mut self) -> Result<Value, OperandStackUnderflow>

Pop a value from the top of the stack

Return OperandStackUnderflow, if no value is available on the stack, which provides an automatic conversion to Effect.

pub fn to_i32_slice(&self) -> &[i32]

Access the stack as a slice of i32 values
pub fn to_u32_slice(&self) -> &[u32]

Access the stack as a slice of u32 values