Value

Struct Value 

pub struct Value { /* private fields */ }
A unit of data

StackAssembly is an untyped languages. All of its values, both on the OperandStack and in Memory, are 32 bits wide. Depending on the situation, they may be interpreted as unsigned or signed.

You can create an instance of Value through its From implementations.

use stack_assembly::Value;

Value::from(3i32);
Value::from(5u32);

Implementations

Source

impl Value

pub fn to_i32(self) -> i32

Convert the value to an i32

Since all values are 32 bits wide, this is always possible. Interprets the bits of the value as a signed (two’s complement) integer.

pub fn to_u32(self) -> u32

Convert the value to a u32

Since all values are 32 bits wide, this is always possible. Interprets the bits of the value as an unsigned integer.

pub fn to_bool(self) -> bool

Convert to a bool

A zero value is considered false, while any other value is considered true.