Memory

Struct Memory 

pub struct Memory {
    pub values: Vec<Value>,
}
A linear memory, freely addressable per word

The memory can be accessed from a script through the read and write operators.

Aside from this, the stack is an important communication channel between script and host. Please refer to Eval’s memory field for more information on that.

Fields

values: Vec<Value>

The values in the memory