pub struct Memory { pub values: Vec<Value>, }
The memory can be accessed from a script through the read and write operators.
read
write
Aside from this, the stack is an important communication channel between script and host. Please refer to Eval’s memory field for more information on that.
Eval
memory
values: Vec<Value>
TypeId
self
Returns the argument unchanged.
Calls U::from(self).
U::from(self)
That is, this conversion is whatever the implementation of From<T> for U chooses to do.
From<T> for U